@@ -1,4 +1,4 @@
-#-*- coding:utf-8 -*-
+# -*- coding:utf-8 -*-
 
 """
 Simple static page generator.
@@ -7,11 +7,13 @@ Uses jinja2 to compile templates.
 """
 
 from __future__ import absolute_import
+from __future__ import print_function
 
 import inspect
 import logging
 import os
 import re
+import shutil
 
 from jinja2 import Environment, FileSystemLoader
 
@@ -32,6 +34,8 @@ class Renderer(object):
                   of the default.
     :param encoding: the encoding of templates to use
     :param logger: a logging.Logger object to log events
+    :param staticpath: the name of the directory to get static files from
+                       (relative to searchpath).
     """
 
     def __init__(self,
@@ -42,6 +46,7 @@ class Renderer(object):
                  logger,
                  contexts=None,
                  rules=None,
+                 staticpath=None
                  ):
         self._env = environment
         self.searchpath = searchpath
@@ -50,6 +55,7 @@ class Renderer(object):
         self.logger = logger
         self.contexts = contexts or []
         self.rules = rules or []
+        self.staticpath = staticpath
 
     @property
     def template_names(self):
@@ -61,6 +67,10 @@ class Renderer(object):
         for template_name in self.template_names:
             yield self.get_template(template_name)
 
+    @property
+    def static_names(self):
+        return self._env.list_templates(filter_func=self.is_static)
+
     def get_template(self, template_name):
         """Get a Template object from the environment.
 
@@ -70,9 +80,9 @@ class Renderer(object):
 
     def _get_context_generator(self, template_name):
         """Get a context generator for a template.
-        
+
         Raises a ValueError if no matching context generator can be found.
-        
+
         :param template_name: the name of the template
         """
         for regex, context_generator in self.contexts:
@@ -103,9 +113,9 @@ class Renderer(object):
 
     def get_rule(self, template_name):
         """Find a matching compilation rule for a function.
-        
+
         Raises a ValueError if no matching rule can be found.
-        
+
         :param template_name: the name of the template
         """
         for regex, render_func in self.rules:
@@ -113,14 +123,32 @@ class Renderer(object):
                 return render_func
         raise ValueError("no matching rule")
 
+    def is_static(self, filename):
+        """Check if a file is a static file (which should be copied, rather
+        than compiled using Jinja2).
+
+        A file is considered static if it lives in the directory
+        specified in ``staticpath``.
+
+        :param filename: the name of the file to check
+
+        """
+        if self.staticpath is None:
+            # We're not using static file support
+            return False
+
+        return filename.startswith(self.staticpath + os.path.sep)
+
     def is_partial(self, filename):
         """Check if a file is a partial.
 
-        Partial files are not rendered, but they are used in rendering templates.
+        Partial files are not rendered, but they are used in rendering
+        templates.
 
         A file is considered ignored if it is prefixed with an ``'_'``.
 
         :param filename: the name of the file to check
+
         """
         return os.path.basename(filename).startswith('_')
 
@@ -143,7 +171,16 @@ class Renderer(object):
 
         :param filename: the name of the file to check
         """
-        return not self.is_partial(filename) and not self.is_ignored(filename)
+        if self.is_partial(filename):
+            return False
+
+        if self.is_ignored(filename):
+            return False
+
+        if self.is_static(filename):
+            return False
+
+        return True
 
     def _ensure_dir(self, template_name):
         """Ensure the output directory for a template exists."""
@@ -153,7 +190,7 @@ class Renderer(object):
             if not os.path.exists(file_dirpath):
                 os.makedirs(file_dirpath)
 
-    def render_template(self, template, context=None):
+    def render_template(self, template, context=None, filepath=None):
         """Render a template.
 
         If a Rule matching the template is found, the rendering task is
@@ -163,6 +200,9 @@ class Renderer(object):
         :param context: optional. A context to render the template with. If no
                         context is provided, `get_context` is used to provide a
                         context.
+        :param filepath: optional. A file or file-like object to dump the
+                         complete template stream into. Defaults to to
+                         ``os.path.join(self.outpath, template.name)``.
         """
         self.logger.info("Rendering %s..." % template.name)
 
@@ -173,28 +213,42 @@ class Renderer(object):
             rule = self.get_rule(template.name)
         except ValueError:
             self._ensure_dir(template.name)
-            fp = os.path.join(self.outpath, template.name)
-            template.stream(**context).dump(fp, self.encoding)
+            if filepath is None:
+                filepath = os.path.join(self.outpath, template.name)
+            template.stream(**context).dump(filepath, self.encoding)
         else:
             rule(self, template, **context)
 
-    def render_templates(self, templates):
+    def render_templates(self, templates, filepath=None):
         """Render a collection of templates.
-        
+
         :param templates: a collection of Templates to render
+        :param filepath: optional. A file or file-like object to dump the
+                         complete template stream into. Defaults to to
+                         ``os.path.join(self.outpath, template.name)``.
         """
         for template in templates:
-            self.render_template(template)
+            self.render_template(template, filepath)
+
+    def copy_static(self, files):
+        for f in files:
+            input_location = os.path.join(self.searchpath, f)
+            output_location = os.path.join(self.outpath, f)
+            print("Copying %s to %s." % (f, output_location))
+            self._ensure_dir(f)
+            shutil.copyfile(input_location, output_location)
 
     def get_dependencies(self, filename):
         """Get every file that depends on a file.
-        
+
         :param filename: the name of the file to find dependencies of
         """
         if self.is_partial(filename):
             return self.templates
         elif self.is_template(filename):
             return [self.get_template(filename)]
+        elif self.is_static(filename):
+            return [filename]
         else:
             return []
 
@@ -204,10 +258,11 @@ class Renderer(object):
         :param use_reloader: if given, reload templates on modification
         """
         self.render_templates(self.templates)
+        self.copy_static(self.static_names)
 
         if use_reloader:
             self.logger.info("Watching '%s' for changes..." %
-                              self.searchpath)
+                             self.searchpath)
             self.logger.info("Press Ctrl+C to stop.")
             Reloader(self).watch()
 
@@ -232,20 +287,20 @@ class Reloader(object):
 
     def should_handle(self, event_type, filename):
         """Check if an event should be handled.
-        
+
         An event should be handled if a file in the searchpath was modified.
 
         :param event_type: a string, representing the type of event
 
         :param filename: the path to the file that triggered the event.
         """
-        print event_type, filename
+        print("%s %s" % (event_type, filename))
         return (event_type == "modified"
                 and filename.startswith(self.searchpath))
 
     def event_handler(self, event_type, src_path):
         """Re-render templates if they are modified.
-        
+
         :param event_type: a string, representing the type of event
 
         :param src_path: the path to the file that triggered the event.
@@ -253,8 +308,12 @@ class Reloader(object):
         """
         filename = os.path.relpath(src_path, self.searchpath)
         if self.should_handle(event_type, src_path):
-            templates = self.renderer.get_dependencies(filename)
-            self.renderer.render_templates(templates)
+            if self.renderer.is_static(filename):
+                files = self.renderer.get_dependencies(filename)
+                self.renderer.copy_static(files)
+            else:
+                templates = self.renderer.get_dependencies(filename)
+                self.renderer.render_templates(templates)
 
     def watch(self):
         """Watch and reload modified templates."""
@@ -263,12 +322,12 @@ class Reloader(object):
 
 
 def make_renderer(searchpath="templates",
-                 outpath=".",
-                 contexts=None,
-                 rules=None,
-                 encoding="utf8",
-                 extensions=None,
-                 ):
+                  outpath=".",
+                  contexts=None,
+                  rules=None,
+                  encoding="utf8",
+                  extensions=None,
+                  staticpath=None):
     """Get a Renderer object.
 
     :param searchpath: the name of the directory to search for templates.
@@ -292,8 +351,12 @@ def make_renderer(searchpath="templates",
 
     :param encoding: the encoding of templates to use. Defaults to ``'utf8'``.
 
-    :param extensions: list of extensions to add to the Environment. Defaults to
-                       ``[]``.
+    :param extensions: list of extensions to add to the Environment. Defaults
+                       to ``[]``.
+
+    :param staticpath: the name of the directory to get static files from
+                       (relative to searchpath). Defaults to ``None``.
+
 
     """
     # Coerce search to an absolute path if it is not already
@@ -318,4 +381,5 @@ def make_renderer(searchpath="templates",
                     logger=logger,
                     rules=rules,
                     contexts=contexts,
+                    staticpath=staticpath,
                     )
